from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class LightBulbInMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/light_bulb_in/v2"
    objects = [
        RigidObjMetaCfg(
            name="bulb0",
            filepath="roboverse_data/assets/rlbench/light_bulb_in/bulb0/usd/bulb0.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="bulb1",
            filepath="roboverse_data/assets/rlbench/light_bulb_in/bulb1/usd/bulb1.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="bulb_holder0",
            filepath="roboverse_data/assets/rlbench/light_bulb_in/bulb_holder0/usd/bulb_holder0.usd",
            physics=PhysicStateType.GEOM,
        ),
        RigidObjMetaCfg(
            name="bulb_holder1",
            filepath="roboverse_data/assets/rlbench/light_bulb_in/bulb_holder1/usd/bulb_holder1.usd",
            physics=PhysicStateType.GEOM,
        ),
        RigidObjMetaCfg(
            name="lamp_base",
            filepath="roboverse_data/assets/rlbench/light_bulb_in/lamp_base/usd/lamp_base.usd",
            physics=PhysicStateType.GEOM,
        ),
    ]
    # TODO: add checker



from __future__ import annotations

from typing import Any, TypeVar

import gymnasium as gym
import numpy as np

from metasim.cfg.scenario import ScenarioMetaCfg
from metasim.types import Action, Extra, Obs

Terminated = TypeVar("Terminated", list[bool])
Truncated = TypeVar("Truncated", list[bool])
RenderFrame = TypeVar("RenderFrame")


class GymEnv(gym.Env):
    def __init__(self, scenario: ScenarioMetaCfg):
        self.scenario = scenario

    ###########################################################
    ## Gym methods
    ###########################################################
    def reset(self, seed: int | None = None, options: dict[str, Any] | None = None) -> tuple[Obs, Extra]:
        pass

    def step(self, action: Action) -> tuple[Obs, float, bool, bool, Extra]:
        pass

    def render(self) -> RenderFrame | list[RenderFrame] | None:
        pass

    def close(self):
        pass

    ###########################################################
    ## Gym properties
    ###########################################################
    ## These properties are not implemented in IsaacLab's DirectRLEnv
    @property
    def action_space(self) -> gym.Space:
        raise NotImplementedError

    @property
    def observation_space(self) -> gym.Space:
        raise NotImplementedError

    @property
    def reward_range(self) -> tuple[float, float]:
        raise NotImplementedError

    @property
    def metadata(self) -> dict[str, Any]:
        raise NotImplementedError

    @property
    def np_random(self) -> np.random.Generator:
        raise NotImplementedError
